package at.ac.uibk.gitsearch.service.dto;

import java.io.Serializable;
import java.util.List;
import java.util.Objects;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

/**
 * A DTO for the {@link at.ac.uibk.gitsearch.domain.Review} entity.
 */
public class ReviewDTO implements Serializable {

    private static final long serialVersionUID = 1L;

    @Size(max = 255)
    private String link;

    @NotNull
    @Size(max = 150)
    private String resource;

    private String resourceID;

    private Long id;
    private List<String> users;
    private List<String> status;
    private List<String> comments;
    private String requestedBy;
    private Boolean badgeRewarded;

    public ReviewDTO() {}

    public ReviewDTO(Long id, String resource, List<String> users, List<String> status, List<String> comments) {
        this.id = id;
        this.resource = resource;
        this.users = users;
        this.status = status;
        this.comments = comments;
    }

    public ReviewDTO(Long id, String resource, List<String> users, List<String> status, List<String> comments, String requestedBy) {
        this.id = id;
        this.resource = resource;
        this.users = users;
        this.status = status;
        this.comments = comments;
        this.requestedBy = requestedBy;
    }

    public ReviewDTO(
        Long id,
        String resource,
        String link,
        List<String> users,
        List<String> status,
        List<String> comments,
        String requestedBy
    ) {
        this.id = id;
        this.resource = resource;
        this.link = link;
        this.users = users;
        this.status = status;
        this.comments = comments;
        this.requestedBy = requestedBy;
    }

    @SuppressWarnings("java:S107") // Methods should not have too many parameters
    public ReviewDTO(
        Long id,
        String resource,
        String link,
        List<String> users,
        List<String> status,
        List<String> comments,
        String requestedBy,
        Boolean badgeRewarded
    ) {
        this.id = id;
        this.resource = resource;
        this.link = link;
        this.users = users;
        this.status = status;
        this.comments = comments;
        this.requestedBy = requestedBy;
        this.badgeRewarded = badgeRewarded;
    }

    public ReviewDTO(
        Long id,
        String resource,
        String link,
        List<String> users,
        List<String> status,
        List<String> comments,
        String requestedBy,
        Boolean badgeRewarded,
        String resourceID
    ) {
        this.id = id;
        this.resource = resource;
        this.link = link;
        this.users = users;
        this.status = status;
        this.comments = comments;
        this.requestedBy = requestedBy;
        this.badgeRewarded = badgeRewarded;
        this.resourceID = resourceID;
    }

    public Boolean getBadgeRewarded() {
        return badgeRewarded;
    }

    public void setBadgeRewarded(Boolean badgeRewarded) {
        this.badgeRewarded = badgeRewarded;
    }

    public String getLink() {
        return link;
    }

    public void setLink(String link) {
        this.link = link;
    }

    public String getRequestedBy() {
        return requestedBy;
    }

    public void setRequestedBy(String requestedBy) {
        this.requestedBy = requestedBy;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getResource() {
        return resource;
    }

    public void setResource(String resource) {
        this.resource = resource;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ReviewDTO)) {
            return false;
        }

        if (this.id == null) {
            return false;
        }
        ReviewDTO reviewDTO = (ReviewDTO) o;
        return Objects.equals(this.id, reviewDTO.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.id);
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "ReviewDTO{" +
            "id=" + getId() +
            ", link='" + getLink() + "'" +
            ", requestedBy=" + getRequestedBy() +
            ", resource='" + getResource() + "'" +
            ", resourceID='" + getResourceID() + "'" +
            "}";
    }

    public List<String> getUsers() {
        return users;
    }

    public void setUsers(List<String> users) {
        this.users = users;
    }

    public List<String> getStatus() {
        return status;
    }

    public void setStatus(List<String> status) {
        this.status = status;
    }

    public List<String> getComments() {
        return comments;
    }

    public void setComments(List<String> comments) {
        this.comments = comments;
    }

    public String getResourceID() {
        return resourceID;
    }

    public void setResourceID(String resourceID) {
        this.resourceID = resourceID;
    }
}
