package at.ac.uibk.gitsearch.service.mapper;

import at.ac.uibk.gitsearch.domain.Authority;
import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.service.dto.ReviewRatingDTO;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;
import java.util.stream.Collectors;
import org.mapstruct.BeanMapping;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingTarget;
import org.mapstruct.Named;
import org.mapstruct.NullValuePropertyMappingStrategy;

/**
 * Mapper for the entity {@link ReviewRating} and its DTO {@link ReviewRatingDTO}.
 */
@Mapper(componentModel = "spring", uses = { ReviewMapper.class, UserMapper.class })
public interface ReviewRatingMapper extends EntityMapper<ReviewRatingDTO, ReviewRating> {
    @Override
    @Mapping(target = "user.authorities", source = "user.authorities", qualifiedByName = "authSet2")
    @SuppressWarnings("PMD.AvoidDuplicateLiterals")
    ReviewRating toEntity(ReviewRatingDTO dto);

    @Override
    @Named("partialUpdate")
    @BeanMapping(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE)
    @Mapping(target = "user.authorities", source = "user.authorities", qualifiedByName = "authSet2")
    void partialUpdate(@MappingTarget ReviewRating entity, ReviewRatingDTO dto);

    @Override
    @Mapping(target = "review", source = "review", qualifiedByName = "resource")
    @Mapping(target = "user", source = "user", qualifiedByName = "id")
    @Mapping(target = "user.authorities", source = "user.authorities", qualifiedByName = "authToStringSet")
    ReviewRatingDTO toDto(ReviewRating s);

    @Named("id")
    @BeanMapping(ignoreByDefault = true)
    @Mapping(target = "id", source = "id")
    ReviewRatingDTO toDtoId(ReviewRating reviewRating);

    @Named("authToStringSet")
    @BeanMapping(ignoreByDefault = true)
    @Mapping(source = "userId.authorities", target = "userId.authorities")
    default Set<String> authoritiesTotrings(Set<Authority> authorities) {
        if (authorities == null) {
            return Collections.emptySet();
        }
        return authorities.stream().map(Authority::getName).collect(Collectors.toSet());
    }

    @Override
    @Named("authSet2")
    @BeanMapping(ignoreByDefault = true)
    @Mapping(target = "user.authorities", source = "user.authorities")
    default Set<Authority> authoritiesFromStrings(Set<String> authoritiesAsString) {
        Set<Authority> authorities = new HashSet<>();

        if (authoritiesAsString != null) {
            authorities =
                authoritiesAsString
                    .stream()
                    .map(string -> {
                        Authority auth = new Authority();
                        auth.setName(string);
                        return auth;
                    })
                    .collect(Collectors.toSet());
        }

        return authorities;
    }
}
