import { Component, OnChanges, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { MultiSelectDropdownExerciseComponent } from 'app/admin/review-management/multi-select-dropdown-exercise/multi-select-dropdown-exercise.component';
import { ReviewManagementService } from 'app/admin/review-management/review-management.service';
import { Review } from 'app/admin/review-management/review.model';
import { ReviewRequest } from 'app/admin/review-management/reviewRequest.model';
import { UserManagementService } from 'app/admin/user-management/service/user-management.service';
import { User } from 'app/admin/user-management/user-management.model';
import { Account } from 'app/core/auth/account.model';
import { AccountService } from 'app/core/auth/account.service';
import { SearchService } from 'app/search/service/search-service';
import { Exercise, searchResultToExercise } from 'app/shared/model/exercise.model';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

@Component({
  selector: 'jhi-reviews',
  templateUrl: './reviews.component.html',
  styleUrls: ['./reviews.component.scss'],
})
export class ReviewsComponent implements OnInit, OnChanges {
  currentAccount: Account | null = null;
  users: User[] | undefined;
  results: Exercise[] = [];
  selectedUsers: string[] | undefined;
  selectedExercise: Exercise | undefined;
  reviews: Review[] = [];
  selectedReview = new Review('', [''], [''], ['']);

  constructor(
    private userService: UserManagementService,
    private accountService: AccountService,
    private searchService: SearchService,
    private reviewManagementService: ReviewManagementService,
    private multiSelectDropdownExerciseComponent: MultiSelectDropdownExerciseComponent,
    private route: ActivatedRoute
  ) {}

  ngOnInit(): void {
    this.accountService.identity().subscribe(account => (this.currentAccount = account));
    this.route.params.subscribe(() => {
      this.loadAll();
    });
  }

  ngOnChanges() {
    this.loadAll();
  }

  loadAll(): void {
    this.userService.loadAll().subscribe(users => {
      this.users = users;
    });
    this.searchService.getAllResources().subscribe(
      (data: SearchResultDTO[]) => {
        const searchResults = data.map(searchResultToExercise);
        this.results = this.results
          .concat(searchResults)
          .filter((value, index, self) => self.findIndex(t => t.title === value.title) === index);
      },
      () => (this.results = [])
    );
    this.reviewManagementService.getAllWithStatistics().subscribe(reviews => {
      this.reviews = reviews;
    });
  }

  shareCheckedUserList(users: string[]) {
    this.selectedUsers = users;
  }
  shareExercise(exercise: Exercise) {
    this.selectedExercise = exercise;
  }

  submitReview() {
    const res = this.selectedUsers!.filter(o => this.selectedExercise?.contributor?.some(x => x.email == o));

    if (res.length > 0) {
      alert('The selected users: ' + res.toString() + ' ,are already contributors of this exercise. Please select other users.');
      return;
    }

    const res2 = this.selectedUsers!.filter(o => this.selectedExercise?.creator.some(x => x.email == o));

    if (res2.length > 0) {
      alert('The selected users: ' + res2.toString() + ' ,are already creators of this exercise. Please select other users.');
      return;
    }

    const res3 = this.selectedUsers!.filter(o => this.selectedExercise?.publisher.some(x => x.email == o));

    if (res3.length > 0) {
      alert('The selected users: ' + res3.toString() + ' ,are already publisher of this exercise. Please select other users.');
      return;
    }

    if (this.reviews.filter(e => e.resource == this.selectedExercise?.title).length > 0) {
      alert('The selected exercise: ' + this.selectedExercise!.title + ' ,is already in the database. Please select other exercise.');
      return;
    }
    this.reviewManagementService
      .create(
        new ReviewRequest(
          this.selectedExercise!.title,
          this.selectedExercise!.originalResult.exerciseId,
          this.selectedUsers!,
          this.selectedExercise!.gitlabURL
        )
      )
      .subscribe(() => {
        this.multiSelectDropdownExerciseComponent.checkedList = [];
        this.loadAll();
      });
  }

  deleteReview(review: Review) {
    this.reviewManagementService.delete(review.id!).subscribe(() => this.loadAll());
  }

  rewardBadge(review: Review) {
    review.status = review.status?.map(() => 'REVIEW_COMPLETED');

    this.reviewManagementService.rewardBadge(review).subscribe(() => this.loadAll());
  }
}
