package at.ac.uibk.gitsearch.repository;

import static org.assertj.core.api.Assertions.assertThat;

import at.ac.uibk.gitsearch.IntegrationTest;
import at.ac.uibk.gitsearch.domain.Statistics;
import at.ac.uibk.gitsearch.repository.jpa.StatisticsRepository;
import java.util.Arrays;
import java.util.List;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

/**
 * Tests for StatisticsRepository
 */
@IntegrationTest
public class StatisticsRepositoryIT {

    @Autowired
    private StatisticsRepository statisticsRepository;

    @Test
    @Transactional
    public void simpleGetByIDTests() {
        for (int i = 0; i < 10; i++) {
            Statistics s = new Statistics();
            s.setExerciseID("id" + i);
            s.setDownloads(100 + i);
            s.setBadgeRewarded(i % 2 == 1 ? true : false);
            s.setViews(1000 + i);
            statisticsRepository.save(s);
        }
        statisticsRepository.flush();

        final List<String> idList = Arrays.asList("id5", "id8");
        final List<Statistics> stats = statisticsRepository.findStatisticsByExerciseIds(idList);
        assertThat(stats).isNotEmpty().hasSize(2);

        final List<String> idList2 = Arrays.asList("id5", "idXXX");
        final List<Statistics> stats2 = statisticsRepository.findStatisticsByExerciseIds(idList2);
        assertThat(stats2).isNotEmpty().hasSize(1);

        final List<String> idList3 = Arrays.asList("id5", "idXXX");
        final List<Statistics> stats3 = statisticsRepository.findStatisticsByExerciseIds(idList3);
        assertThat(stats3).isNotEmpty().hasSize(1);
    }
}
